import com.threed.jpct.*;

public class CalcHelper{

	public static float getDistance(SimpleVector vt, SimpleVector vs) throws Exception{
		float flen = 0f;
		try{
			float fx = vt.x - vs.x;
			float fy = vt.y - vs.y;
			float fz = vt.z - vs.z;
			 
			flen = (float)Math.sqrt(fx*fx + fy*fy + fz*fz);

		}catch(Exception e){
			System.err.println("CalcHelper::getDistance() " + e.getMessage());
			e.printStackTrace();
			throw e;
		}
		return flen;

	}

	public static SimpleVector getMouseVector(World world, Camera camera, FrameBuffer buffer, int x, int y){
       SimpleVector vret = null;
	   SimpleVector ray=Interact2D.reproject2D3D(camera, buffer, x, y);
       if (ray!=null) {
          SimpleVector vnorm=ray.normalize();
          Matrix mat=camera.getFront();
          mat=mat.invert3x3();
          vnorm.matMul(mat);
 
          float f=world.calcMinDistance(camera.getPosition(), vnorm, 1000);
          if (f!=Object3D.COLLISION_NONE) {
             // copy the normal vector so we can subtract it out after altering the magnitude...
			 SimpleVector voffset=new SimpleVector(vnorm);
             vnorm.scalarMul(f);
             vnorm=vnorm.calcSub(voffset);
             
			 SimpleVector vcam = camera.getPosition();
			 
			 //the camera is set back a large negative y and z (ie. (0, -160, -100).  
			 //vnorm right now reflects the full magnitude of the camera
			 //ray as a postive (x, 159, 43).  Adding the vectors removes the camera's magnitude
			 //since they share the same normal direction. 
			 vret = new SimpleVector(vcam);
			 vret.add(vnorm);
          }
       }
	   return vret;
    }

	   public static float rotateToTarget(Object3D osource, Object3D otarget){
   		float yRot = 0;
   		try{
	      SimpleVector xd=osource.getTransformedCenter();
	      xd.scalarMul(-1f);

		  SimpleVector target = otarget.getTransformedCenter();
	  
		  // normalize the y axis between the objects to avoid x tilt...
		  target.add(new SimpleVector(0, (xd.y + target.y)*-1, 0));	  

	      xd.add(target);
      
	      // Copy it...
	      Matrix rotY=xd.getRotationMatrix();
      
	      //this.setRotationMatrix(_rotY);
			
  		  // this measures (PI/2 radians) -1.5707 to 1.5707 aka -180 to 180 degrees on x...  
          float[] md =rotY.getDump();
          float rad =(float) Math.asin(-md[2]);

		  // this will give us x and z rotations around y...
		  SimpleVector vz = rotY.getZAxis();	
		  
		  float fPi = 0f;	

          // once we know which 90 degree quadrant we are in 
		  // we can adjust the raidians and add to magnitudes of PI to get 0 - 6.28... PI
		  if(vz.x > 0 && vz.z > 0)
		  	fPi = (float)rad;
		  else if(vz.x > 0 && vz.z < 0)
		  	fPi = (float)Math.PI - (float)rad;
		  else if(vz.x < 0 && vz.z < 0)
		  	fPi = (float)Math.PI - (float)rad;
          else if(vz.x < 0 && vz.z > 0)
		  	fPi = (float)Math.PI*2  + (float)rad;
		  		  
		  // lets try it!
	      Matrix rotMat = osource.getRotationMatrix();
	      rotMat.setIdentity();
	      osource.setRotationMatrix(rotMat);
	      osource.rotateY(0);
		  osource.rotateY(fPi);
		  yRot = fPi;

		}catch(Exception e){
			System.err.println("CalcHelper::rotateToTarget() " + e.getMessage());
			e.printStackTrace();
		}
		return yRot;	
   }

   


}