

import java.awt.Color;
import com.threed.jpct.*;
import com.threed.jpct.util.*;
import com.threed.jpct.procs.*;
import java.util.*;


public class FocalPoint {
	public static final String DEFAULT_OBJECT = "_FP.default";
	public static final String FP_PREFIX = "_FP.";
	private HashMap _hsFP = null;
	private World _world = null;
	private boolean _selected = false;	

	public boolean up = false;
	public boolean down = false;
	public boolean left = false;
	public boolean right = false;
	public boolean forward = false;
	public boolean backward = false;
	public boolean turn_left = false;
	public boolean turn_right = false;


	class FPAttributes{
		public float yrot = 0f;
		public float xrot = 0f;
		public float zrot = 0f;
		Object3D obj = null;
		String name = null;
	}

	public void checkMoveSelected(){
		try{
			
			Object3D ofp = getObject3D(DEFAULT_OBJECT);
			SimpleVector vnorm = ofp.getZAxis();
	
			SimpleVector vnew = new SimpleVector(0f, 0f, 0f);		
			if(up)
				vnew.y -= 1;
			if(down)
				vnew.y += 1;
			if(left)
				vnew.x -= 1;
			if(right)
				vnew.x += 1;	
			if(forward){
				vnew = new SimpleVector(vnorm.x, 0f, vnorm.z);
				vnew.scalarMul(4f);
			}
			if(backward){
				vnew = new SimpleVector(vnorm.x, 0f, vnorm.z);
				vnew.scalarMul(-4f);
			}

			if(turn_left)
				ofp.rotateY(-0.075f);

			if(turn_right)
				ofp.rotateY(0.075f);

			SimpleVector vcoll = ofp.checkForCollisionEllipsoid(vnew, new SimpleVector(5f,10f,5f), 5);
			ofp.translate(vcoll);

		}catch(Exception e){
			System.out.println("FocalPoint::checkMoveSelected() " + e.getMessage());
			e.printStackTrace();
		}
	}

	
	private FPAttributes getFPAttributes(String sname){
		FPAttributes fp = null;
		try{
			fp = (FPAttributes)_hsFP.get(sname);
			System.out.println("FocalPoint::getFPAttributes() fp is null: " + (fp == null));
		}catch(Exception e){
			System.out.println("FocalPoint::getFPAttributes() " + e.getMessage());
			e.printStackTrace();
		}
		return fp;
	}

	public Object3D getObject3D(String sname){
		Object3D o = null;
		try{
			if(_hsFP.containsKey(sname)){
				FPAttributes fp = (FPAttributes)_hsFP.get(sname);
				o = fp.obj;
			}

		}catch(Exception e){
			System.out.println("FocalPoint::getObject3D() " + e.getMessage());
			e.printStackTrace();
		}
		return o;
	}
	
	public Object3D addFocalPoint(String sname){
		Object3D o = null;
		try{
			if(!_hsFP.containsKey(sname)){
				FPAttributes fp = new FPAttributes();

				String sfile = DevkitConfig.getInstance().getValue(DevkitConfig.DEVKIT_MODELS_PATH)
								+ "/camera.3DS";

				System.out.println("FocalPoint::addFocalPoint() loading [" + sfile + "]");

				Object3D[] oa=Loader.load3DS(
					getClass().getClassLoader().getResourceAsStream(sfile), 2f);
		
				fp.obj = oa[0];
				fp.name = sname;
				fp.obj.build();

				fp.obj.rotateX((float)-Math.PI/3);

				fp.obj.translate(new SimpleVector(0f, -40f, 0f));
				_world.addObject(fp.obj);
				fp.obj.setTexture("camera.jpg");
				
				fp.obj.setCollisionMode(Object3D.COLLISION_CHECK_SELF|Object3D.COLLISION_CHECK_OTHERS); 

				fp.obj.setName(sname);
				o = fp.obj;
				_hsFP.put(fp.obj.getName(), fp);

			}

		}catch(Exception e){
			System.out.println("FocalPoint::getObject3D() " + e.getMessage());
			e.printStackTrace();
		}
		return o;

	}

	
	public void placeFocalPoint(SimpleVector v){
		try{
			Object3D fp = getObject3D(DEFAULT_OBJECT);
			SimpleVector vtc = fp.getTransformedCenter();

			vtc.scalarMul(-1);
			fp.translate(vtc);

			v.y -= 10;
			fp.translate(v);

		}catch(Exception e){
			System.err.println("FocalPoint::placeFocalPoint() " + e.getMessage());
			e.printStackTrace();
		}	
	}
	

	public void resetToTerrain(Object3D terrain){
		try{
			Object3D fp = getObject3D(DEFAULT_OBJECT);
			SimpleVector vtc = fp.getTransformedCenter();
			vtc.y -= 5000;

			fp.translate(vtc);

			SimpleVector vdist = new SimpleVector(0, 1, 0);
		
			float fdist = terrain.calcMinDistance(fp.getTransformedCenter(), vdist);
	
			System.out.println("FocalPoint::resetToTerrain() fdist: " + fdist);

			if (fdist != Object3D.COLLISION_NONE) {
				
				vdist.scalarMul(fdist - 150);

				System.out.println("FocalPoint::resetToTerrain() moving vdist.y: " + vdist.y);
				
				vtc = fp.getTransformedCenter();

				System.out.println("FocalPoint::resetToTerrain() vtc.y: " + vtc.y);
				System.out.println("FocalPoint::resetToTerrain() vdist.y: " + vdist.y);

				fp.translate(vdist);

				vtc = fp.getTransformedCenter();
				System.out.println("FocalPoint::resetToTerrain() new vtc.y: " + vtc.y);


			}


		}catch(Exception e){
			System.err.println("FocalPoint::resetToTerrain() " + e.getMessage());
			e.printStackTrace();
		}
	}

	public void init(World w){
		try{
			_world = w;
			_hsFP = new HashMap();
			addFocalPoint(DEFAULT_OBJECT);

		}catch(Exception e){
			System.err.println("FocalPoint::init() " + e.getMessage());
			e.printStackTrace();
		}

	}

}